;============================================================================
; DBUPD.ASM
;
; Database module - database update thread.
;
; NB The subsidiary routines and the variables declared below are for
; UpdateDB use ONLY.
;============================================================================




; Database polling interval.
;
; Time between wait objects to allow for the exit flag to be picked up.

UDBWait1Sec	equ	1*1000		; 1 second, in mS




.data




; Variables local to UDB only.
;
; These are summing or shared variables used in the recursive functions so
; they need to be stored externally.

nPathsFiles	dd	0		; number of Files and Paths
pDB		dd	0		; pointer into Database
PathNdx		dd	0		; Path Index

bRecycle	dd	0		; true if in a recycle bin directory
sRecycle	db	"RECYCLED"	; recycle bin detection
NsRecycle	dd	8		; #char in sRecycle

TmpFile		MMF	<offset szTmpFile,0,GENERIC_READ or GENERIC_WRITE, \
			CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0,0,0>

		align	dword




.code




;----------------------------------------------------------------------------
UpdateDB	proc	tParam:dword
;-------
;
; Database update thread.
;
;
; Entry		tParam		n/a
;
;
; External	bUDBExit	Exit requested, if set then abort
;
;		UDBStatus	Status:
;
;				UDBCntFS	Counting files on all drives
;				UDBWaitFS	Waiting for filesystem change
;				UDBCreate	Creating database
;				UDBWaitUP	Waiting for update permission
;				UDBUpd		Updating database
;				UDB1stDone	Initial update finished
;				UDBDbStale	Database is stale - change
;						detected but update not done
;
;
; Notes
;
; The temporary file is sized and created at the start of every session so
; the size of an existing database has no bearing on the new size.
;
; At first entry a scan is always done, triggered by UDB1stDone bit clear.
;
; The update tries to replace the Search array in the state it was before the
; update happened so the user does not see too much of a change in the Time
; window display. Works most of the time!.
;----------------------------------------------------------------------------


	local	szRoot[4]		:byte	; Drive root path
	local	DriveSpec[32]		:byte	; Drive letters, ascending order
	local	hDrvChange[32]		:dword	; Drive change notif. handles
	local	NumDrives		:dword	; number of Drives found
	local	pTmpFile		:dword	; pointer, mapped temp file
	local	bTmpFile		:dword	; flag, temp file created
	local	NewDBSize		:dword	; Updated DB size, bytes
	local	bUpdate			:dword	; flag, do update
	local	OrgFI			:dword	; orig first item index
	local	OrgCI			:dword	; orig current item index
	local	OrgNI			:dword	; orig number of items


	if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_UDBEntry
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


; Clear local variables, reset externals.


	mov	ecx, ebp
	mov	edi, esp
	sub	ecx, edi
	xor	eax, eax
	rep	stosb

	mov	bUDBExit, eax			; FALSE
	mov	G_UDBStatus, eax		; clear status
	or	G_UDBStatus, UDBDbStale		; stale until first update


; Get all fixed Drives on the system and create change notification objects
; for each of them.


	if HDC_Only	;++++++++++++++++++++++++++++++++++++++++++++++++++++

		xor	eax, eax
		mov	al, 'C'

		if	HDD_SubstD		; Substitute D as has very
			mov	al, 'D'		;  few files on my system.
		endif				; Makes testing very quick.

		mov	DriveSpec[0], al
		mov	DriveSpec[1], ah

		add	eax, '\:' shl 8
		mov	dword ptr szRoot, eax

		xor	ebx, ebx

		invoke	FindFirstChangeNotification, addr szRoot,
				TRUE,
				FILE_NOTIFY_CHANGE_FILE_NAME or \
				FILE_NOTIFY_CHANGE_DIR_NAME

		mov	hDrvChange[ebx*4], eax
		inc	ebx

	else		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	; Normal operation - get all drives.


	invoke	GetLogicalDrives

	mov	ecx, eax
	shr	ecx, 2				; Ignore floppy drives A, B

	mov	eax, '\:C'			; 'C:\',0 - start with C:
	mov	dword ptr szRoot, eax

	xor	ebx, ebx

	.while	(ecx > 0)

		shr	ecx, 1			; Next drive flag
		push	ecx

		invoke	GetDriveTypeA, addr szRoot

		; If fixed drive then add it.

		.if	(eax == DRIVE_FIXED)

			; Load drive letter to string.

			mov	al, szRoot[0]
			mov	[DriveSpec + ebx], al

			; Create root directory notification object.
			; It watches all subdirectories for file or
			; directory name change (add/delete/rename).

			invoke	FindFirstChangeNotification, addr szRoot,
					TRUE,
					FILE_NOTIFY_CHANGE_FILE_NAME or \
					FILE_NOTIFY_CHANGE_DIR_NAME

			mov	hDrvChange[ebx*4], eax

			inc	ebx

		.endif

		; Next drive letter.

		inc	szRoot[0]
		pop	ecx

	.endw

	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	xor	eax, eax
	mov	[DriveSpec + ebx], al		; add Drive list terminator
	mov	NumDrives, ebx			; number of drives found


; Count total of files and paths so we can set the temporary file size.
;
; This takes the bulk of the time when first creating a new database however
; as the find history seems to be stored by Windows(?), when we scan again
; below to do the database creation it goes much faster.


	or	G_UDBStatus, UDBCntFS

	xor	eax, eax
	mov	nPathsFiles, eax

	mov	ah, ':'
	mov	dword ptr szRoot, eax		; '<drv>:',0

	xor	ebx, ebx

	.while	(ebx < NumDrives)

		mov	al, [DriveSpec + ebx]	; next drive
		mov	szRoot[0], al

		invoke	CountPath, addr szRoot	; (recursively)

		.if	(bUDBExit)		; Exit requested
			jmp	@@Exit
		.endif

		inc	ebx

	.endw

	and	G_UDBStatus, not UDBCntFS


; Estimate the space required. Empirically ~30 bytes per entry was found
; so setting to 64 doubles the size - we should have no problems unless
; the user doubles the number of files on their system in this session!.
;
; Create the temporary file for this session and memory map it.


	mov	eax, nPathsFiles
	shl	eax, 6				; *64
	mov	TmpFile.nFile, eax

	invoke	File_Map, addr TmpFile

	mov	eax, TmpFile.pMFile
	mov	pTmpFile, eax

	mov	bTmpFile, 1			; Temp file now created


; Worker thread loop ========================================================
;
; Loop until instructed to exit.


.while	TRUE


	; Check for exit request.

	.break	.if (bUDBExit)


	; If first entry then bypass the change notification.

	.if	!(G_UDBStatus & UDB1stDone)
		jmp	@@CreateDB
	.endif


	; Initialise the notification for each drive.

	xor	ebx, ebx

	.while	(ebx < NumDrives)

		mov	eax, hDrvChange[ebx*4]
		invoke	FindNextChangeNotification, eax
		inc	ebx

	.endw


	; Wait for a change in any drive's contents.

	or	G_UDBStatus, UDBWaitFS


	if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_UDBWaitChg
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	.while	TRUE

		.if	(bUDBExit)		; Exit requested
			jmp	@@Exit
		.endif

		invoke	WaitForMultipleObjects, NumDrives, addr hDrvChange,
				FALSE, UDBWait1Sec

		.break .if (eax != WAIT_TIMEOUT)

	.endw

	and	G_UDBStatus, not UDBWaitFS


	; Create the new database.

	@@CreateDB:
	;----------


	or	G_UDBStatus, UDBCreate
	or	G_UDBStatus, UDBDbStale


	if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_UDBStCre
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	; Scan paths and files into Database.

	mov	eax, pTmpFile			; initialise pointer
	mov	pDB, eax
	mov	edi, eax			; start of Database

	xor	eax, eax
	mov	ah, ':'
	mov	dword ptr szRoot, eax		; '<drv>:',0

	xor	ebx, ebx
	mov	PathNdx, -1

	.while	(ebx < NumDrives)

		mov	al, [DriveSpec + ebx]	; next drive
		mov	szRoot[0], al

		invoke ScanPath, addr szRoot	; (recursively)

		inc	ebx

	.endw


	; Check for exit request.

	.break	.if (bUDBExit)


	; Add terminator record (record Length field is 0)
	; and store Database size in bytes.

	xor	eax, eax
	mov	edi, pDB
	stosd

	mov	eax, edi
	sub	eax, pTmpFile
	mov	NewDBSize, eax

	and	G_UDBStatus, not UDBCreate


	if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_UDBEnCre
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	; Wait until an update happens, polling the Event object to
	; see if the Time window is down. If it is then we can update
	; the Database.

	or	G_UDBStatus, UDBWaitUP
	mov	bUpdate, TRUE

	.while	(bUpdate)


		.if	(bUDBExit)		; Exit requested
			jmp	@@Exit
		.endif


		; Wait for the Event object to be signalled and update the
		; database if we haven't already done it. If the event wait
		; times out then go round power required.

		invoke	WaitForSingleObject, G_UDBhEvent, UDBWait1Sec


		.if	((eax == WAIT_OBJECT_0) && (bUpdate))


			; Update Database ***********************************


			or	G_UDBStatus, UDBUpd
			and	G_UDBStatus, not UDBWaitUP


			if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++
					.data
					Dbg_UDBUpd	dd	0
					Dbg_UDBCnt	dd	0 ; seconds
					.code
					mov	Dbg_UDBUpd, 1
					mov	Dbg_UDBCnt, 0
			endif		;++++++++++++++++++++++++++++++++++++


			; Grab the update mutex.

			invoke	WaitForSingleObject, G_UDBhMutex, UDBWait1Sec


			; Create / alter Database memory image.

			mov	edx, DBSize
			mov	eax, NewDBSize

			.if	(!pDataBase)

				; New database so allocate to new size
				; and flag allocation done.

				invoke	Mem_Alloc, eax
				mov	pDataBase, eax

			.elseif	(eax > edx)

				; Need to increase memory image size.

				invoke	Mem_ReAlloc, addr pDataBase, eax

			.endif


			; Copy mapped temp file database over existing
			; database image and update Database size.

			mov	esi, pTmpFile
			mov	edi, pDataBase
			mov	ecx, NewDBSize
			mov	DBSize, ecx
			shr	ecx, 2		; dword move
			cld
			rep	movsd


			; Delete original arrays and create anew.

			invoke	BuildArrays


			; Replace the last search.
			;
			; Get the Search array prior display state.

			invoke	Time_GetItem
			mov	OrgFI, eax	; first item index
			mov	OrgCI, ecx	; current item index
			mov	OrgNI, edx	; nItems

			; Redo the search on the new database.

			invoke	DB_Search, LS_Flags, addr LS_Text, LS_LenText
			mov	ebx, eax

			; Set Time display to prior state.
			;
			; Set the original first item as the start item.
			; If it is greater than then number of items
			; then reset it as the first item.

			mov	edx, OrgFI

			.if	(edx >= ebx)
				xor	edx, edx
			.endif

			invoke	Time_SetItem, TSetFirst, edx, ebx

			; Set the number of items to the lesser of
			; the original or the new number. The original
			; will be 0 on first entry so load local copy.

			mov	edx, OrgNI

			.if	(!edx)
				mov	edx, LS_FileCnt
			.endif

			mov	OrgNI, edx

			.if	(edx < ebx)
				mov	ebx, edx
			.endif

			invoke	Time_SetItem, TSetFirst, 0, ebx

			; Reset the original current item as first item
			; in the display.

			mov	eax, OrgCI

			.if	(eax >= ebx)		; If past end
				mov	eax, ebx	;  then reset
				dec	eax		;  to last item.
			.endif

			invoke	Time_SetItem, TSetItem, eax, ebx


			; Update done, exit from the update loop and go back
			; to polling for a filesystem change.

			mov	bUpdate, FALSE
			and	G_UDBStatus, not UDBUpd
			and	G_UDBStatus, not UDBDbStale
			or	G_UDBStatus, UDB1stDone
			invoke	ReleaseMutex, G_UDBhMutex


			;****************************************************


		.endif


		if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_UDBWaitUpd, Dbg_UDBCnt, Dbg_UDBUpd
				mov	Dbg_UDBUpd, 0
				inc	Dbg_UDBCnt
		endif		;++++++++++++++++++++++++++++++++++++++++++++


	.endw

.endw


@@Exit:


; Exit thread ===============================================================


	if Dbg_Msgs	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_UDBExit
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


; Release change notification objects.


	xor	ebx, ebx

	.while	(ebx < NumDrives)

		mov	eax, hDrvChange[ebx*4]
		invoke	FindCloseChangeNotification, eax
		inc	ebx

	.endw


; Release and delete temporary file.


	.if	(bTmpFile)

		mov	TmpFile.Mode, TRUE		; delete
		invoke	File_UnMap, addr TmpFile

	.endif


; Flag the thread is exiting and exit.


	mov	bUDBExit, 0

	invoke	ExitThread, 0


UpdateDB	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
CountPath	proc	pszPath:dword
;--------
;
; Recursive count of all files and directories in a path.
;
;
; Entry		pszPath		pointer to asciiz path
;
;		External:
;
;		nPathsFiles	0 at first entry or prior count
;
;
; Working	bUDBExit	flag, if set then abort recursively so we
;				release the Find history
;
;
; Exit		nPathsFiles	count of paths and files found
;
;
; Registers	preserved
;
;
; Errorhandling
;
; If FindFirst or FindNext fail they return 0. If no files are found they
; return ERROR_NO_MORE_FILES (= 18) and it seems wfd.cFileName <- 0. Thus
; if an error actually occurs the examination of cFileName will give a 0 so
; the function will safely exit.
;----------------------------------------------------------------------------


	local	wfd		:WIN32_FIND_DATA
	local	pPath[MAX_PATH]	:byte			; parent path
	local	cPath[MAX_PATH]	:byte			; child path
	local	hFind		:dword			; find handle


	pushad


; Store a local copy of the parent path appending the wildcard
; specification '\*' to it.


	lea	edi, pPath
	invoke	szCopy, pszPath, edi		; (returns char copied)
	add	edi, eax
	mov	ax, '*\'
	mov	[edi], ax
	xor	eax, eax
	mov	[edi + 2], al			; terminator


; Count the path.


	inc	nPathsFiles


; Recurse path.


	xor	eax, eax
	mov	dword ptr wfd.cFileName, eax	; set FileName to NULL

	invoke	FindFirstFileA, addr pPath, addr wfd

	mov	hFind, eax
	mov	ebx, eax

	.while	(ebx > 0)

		lea	esi, wfd.cFileName
		mov	eax, [esi]

		.if	((eax != '..') && (eax != '.'))

			mov	eax, wfd.dwFileAttributes

			.if eax & FILE_ATTRIBUTE_DIRECTORY

				; Make a new search path by concatenating the
				; parent path and the directory found. This
				; forms the child path. The wildcard '*' in
				; the parent path is replaced with the zero
				; terminator for szCatStr.

				lea	edi, cPath
				invoke	szCopy, addr pPath, edi
				mov	byte ptr [edi + eax - 1], 0
				invoke	szCatStr, edi, esi

				; Check for exit requested.

				.if	(bUDBExit)
					jmp	@@Exit
				.endif

				; Recursive count.

				invoke	CountPath, edi

			.else

				; File, count it.

				inc	nPathsFiles

			.endif

		.endif

		invoke	FindNextFileA, hFind, addr wfd
		mov	ebx, eax

	.endw


	@@Exit:

	invoke	FindClose, hFind


	popad


	ret


CountPath	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
ScanPath	proc	pszPath:dword
;-------
;
; Recursive scan of a path, loading paths and files into Database.
;
; First scan for files in a directory before scanning any subdirectories.
; This gives all the files in a directory before starting on a subdirectory.
; The normal recursive search method will recurse into subdirectories before
; finishing with a parent.
;
;
; Entry		pszPath		pointer to asciiz path
;
;		pDB		ptr, current Database record
;
;
; Working	bUDBExit	flag, if set then abort recursively so we
;				release the Find history
;
;
; Exit		pDB		next Database record
;
;
; Registers	preserved
;
;
; Errorhandling
;
; If FindFirst or FindNext fail they return 0. If no files are found they
; return ERROR_NO_MORE_FILES (= 18) and it seems wfd.cFileName <- 0. Thus
; if an error actually occurs the examination of cFileName will give a 0 so
; the function should safely exit.
;----------------------------------------------------------------------------


	local	wfd		:WIN32_FIND_DATA
	local	pPath[MAX_PATH]	:byte			; parent path
	local	cPath[MAX_PATH]	:byte			; child path
	local	hFind		:dword			; find handle


	pushad


; Add the path to the database.


	invoke	AddPath, pszPath


; Store a local copy of the parent path, appending the wildcard
; specification '\*' to it.


	lea	edi, pPath
	invoke	szCopy, pszPath, edi		; (returns char copied)
	add	edi, eax
	mov	ax, '*\'
	mov	[edi], ax
	xor	eax, eax
	mov	[edi + 2], al			; terminator


; Add all files in the current path.


	xor	eax, eax
	mov	dword ptr wfd.cFileName, eax	; set FileName to NULL

	invoke	FindFirstFileA, addr pPath, addr wfd
	mov	hFind, eax

	.while	(eax > 0)

		lea	esi, wfd.cFileName
		mov	eax, [esi]

		.if	((eax != '..') && (eax != '.'))

			mov	eax, wfd.dwFileAttributes

			.if	!(eax & FILE_ATTRIBUTE_DIRECTORY)

				invoke	AddFile, addr wfd

			.endif

		.endif

		invoke	FindNextFileA, hFind, addr wfd

	.endw

	invoke	FindClose, hFind


; Find all subdirectories in the current path and recurse each of them.


	xor	eax, eax
	mov	dword ptr wfd.cFileName, eax	; set FileName to NULL

	invoke	FindFirstFileA, addr pPath, addr wfd
	mov	hFind, eax

	.while	(eax > 0)

		lea	esi, wfd.cFileName
		mov	eax, [esi]

		.if	((eax != '..') && (eax != '.'))

			mov	eax, wfd.dwFileAttributes

			.if	(eax & FILE_ATTRIBUTE_DIRECTORY)

				; Check for recycle bin directory.

				invoke	StrLen, esi
				mov	edx, eax
				invoke	InStrgI, 0, esi, edx, addr sRecycle,
						NsRecycle

				.if	(!CARRY?)
					xor	eax, eax
					inc	eax
				.else
					xor	eax, eax
				.endif

				mov	bRecycle, eax

				; Make a new search path by concatenating the
				; parent path and the directory found. This
				; forms the child path. The wildcard '*' in
				; the parent path is replaced with terminator
				; for szCatStr.

				lea	edi, cPath
				invoke	szCopy, addr pPath, edi
				mov	byte ptr [edi + eax - 1], 0
				invoke	szCatStr, edi, esi

				; Check for exit requested.

				.if	(bUDBExit)
					jmp	@@Exit
				.endif

				; Recursive scan.

				invoke	ScanPath, edi

			.endif

		.endif

		invoke	FindNextFileA, hFind, addr wfd

	.endw


	@@Exit:

	invoke	FindClose, hFind


	popad


	ret


ScanPath	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
AddPath		proc	pszPath:dword
;------
;
; Add a Path record to the database, appending '\'.
;
; NB only to be called from ScanPath.
;
;
; Entry		pszPath		pointer to asciiz path
;
;		External:
;
;		pDB		ptr, current Database record
;		PathNdx		previous Path index
;
;
; Exit		pDB		next Database record
;		PathNdx		current Path index
;
;
; Registers	preserved
;----------------------------------------------------------------------------


	pushad


; Construct Path record spec and append path.


	inc	PathNdx

	mov	edi, pDB		; start of record
	mov	ebx, edi		; save spec pointer

	mov	eax, PathNdx		; align Path index
	shl	eax, RecPNdxShft
	or	eax, RecPathFlag	; add Path record flag

	stosd				; and store

	invoke	szCopy, pszPath, edi	; copy path over
	add	edi, eax		; (returns char copied)
	mov	ax, '\'
	stosw				; append '\',0


; Pad record to dword boundary.


	mov	ecx, 011b
	test	edi, ecx
	jz	@@NoPad

	mov	eax, edi
	and	eax, ecx
	inc	ecx
	sub	ecx, eax
	xor	eax, eax
	rep	stosb

	@@NoPad:


; Insert length into specification field.


	mov	pDB, edi		; start of next record

	sub	edi, ebx		; length of this record
	add	[ebx], edi		; insert


	popad


	ret


AddPath		endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
AddFile		proc	pwfd:dword
;------
;
; Add a File record to the database.
;
; NB only to be called from ScanPath.
;
;
; Entry		pwfd		pointer to WIN32_FIND_DATA structure
;
;		External:
;
;		pDB		ptr, current Database record
;		PathNdx		current Path index
;		bRecycle	flag, recycle bin directory
;
;
; Exit		pDB		next Database record
;		PathNdx		unchanged
;		bRecycle	unchanged
;
;
; Registers	preserved
;----------------------------------------------------------------------------


	local	ftl		:FILETIME	; file localised time
	local	fts		:SYSTEMTIME	; file system time
	local	IPath[MAX_PATH]	:byte		; full path to file
	local	sfx[8]		:byte		; suffix assembly


	pushad


; Record Specification field.


	mov	edi, pDB			; start of record
	push	edi				; save spec pointer

	mov	eax, PathNdx			; align Path index
	shl	eax, RecPNdxShft

	stosd					; and store


; File Specification field.


	; Extract the file suffix as a value for comparison
	; with the little-endian values in the Suffix List.

	mov	esi, pwfd
	add	esi, WIN32_FIND_DATA.cFileName
	xor	ecx, ecx
	mov	edx, ecx
	dec	ecx
	mov	ah, '.'

	; Scan for the last '.' in the name string.

	@@ScanName:
	inc	ecx
	mov	al, [esi + ecx]

	.if	(al == ah)			; found a '.'
		mov	edx, ecx		; mark its position
	.endif

	or	al, al				; terminator?
	jnz	@@ScanName			; no

	; End of scan, edx may have a '.' position and ecx has position
	; of terminator. Check for boundary conditions.

	.if	(edx == 0)

		; No '.' in name, take last 4 or all if < 4 char.

		sub	ecx, 4

		.if	(sdword ptr ecx > 0)
			add	esi, ecx
		.endif

	.else

		; We have a '.' so move to it. However if there
		; are >= 4 char following it then move past the '.'

		mov	eax, ecx		; end position
		sub	ecx, edx

		.if	(ecx > 4)		; > 4 char past '.'
			sub	eax, 4		; so take last 4
			mov	edx, eax
		.endif

		add	esi, edx

	.endif

	; Uppercase it, loading buffer + terminating and padding 0's.
	; [esi] has beginning of suffix.

	push	edi

	lea	edi, sfx
	mov	dx, 'za'

	@@LoadSfx:
	mov	al, [esi]
	or	al, al			; End of name?
	jz	@@SfxDone		; yes
	cmp	al, dl
	jb	@@UCDone
	cmp	al, dh
	ja	@@UCDone
	sub	al, 32
	@@UCDone:
	mov	[edi], al
	inc	edi
	inc	esi
	jmp	@@LoadSfx

	@@SfxDone:

	movzx	eax, al
	mov	[edi], eax		; ensure no garbage

	pop	edi

	; Match the file suffix with a known suffix if possible
	; and load specification for the file.

	mov	ebx, dword ptr [sfx]
	mov	esi, pSfxList
	mov	ecx, NumSfx

	@@NextSfx:
	mov	eax, [esi + SFX.FSpec]
	cmp	ebx, [esi + SFX.Sfx]
	je	@@LoadSpec

	add	esi, sizeof SFX
	dec	ecx
	jnz	@@NextSfx

	mov	eax, FG_Unk			; unknown

	@@LoadSpec:

	.if	(bRecycle)

		; If we are in a recycle bin then that is
		; all the spec we need.

		mov	eax, FG_Del
		or	eax, RecRecyFlag

	.endif

	stosd					; store


; File Suffix field.


	mov	eax, ebx
	stosd


; TimeStamp field.


	; Convert the file time to local time and then to
	; system time, y/m/d/h/s.

	mov	edx, pwfd
	add	edx, WIN32_FIND_DATA.ftLastWriteTime
	invoke	FileTimeToLocalFileTime, edx, addr ftl
	invoke	FileTimeToSystemTime, addr ftl, addr fts


	; Pack to Time format and store.
	;
	; b31..b26	6 bits	1980..2044
	; b25..b22	4 bits	month 0..11
	; b21..b17	5 bits	day 0..30
	; b16..b12	5 bits	hour 0..23
	; b11..b6	6 bits	minutes 0..59
	; b5..b0	6 bits	seconds 0..59

	mov	ax, fts.wYear
	sub	ax, 1980
	shl	eax, 26
	mov	edx, eax
	movzx	eax, fts.wMonth
	shl	eax, 22
	add	edx, eax
	movzx	eax, fts.wDay
	shl	eax, 17
	add	edx, eax
	movzx	eax, fts.wHour
	shl	eax, 12
	add	edx, eax
	movzx	eax, fts.wMinute
	shl	eax, 6
	add	edx, eax
	movzx	eax, fts.wSecond
	add	eax, edx

	stosd					; and store


; File name field.


	mov	edx, pwfd
	add	edx, WIN32_FIND_DATA.cFileName
	invoke	szCopy, edx, edi		; (returns count)
	inc	eax				; terminating zero


; Pad record to dword boundary.


	mov	ecx, 011b
	add	edi, eax
	test	edi, ecx
	jz	@@NoPad

	mov	eax, edi
	and	eax, ecx
	inc	ecx
	sub	ecx, eax
	xor	eax, eax
	rep	stosb

	@@NoPad:


; Insert length into specification field.


	mov	pDB, edi			; start of next record

	pop	eax				; start of record
	sub	edi, eax			; length of this record
	add	[eax], edi			; insert


	popad


	ret


AddFile		endp
;----------------------------------------------------------------------------




;============================================================================
