;============================================================================
; TIMESHOW.ASM
;
; Time module - ShowTime and its subsidiary function DrawItem.
;
; ShowTime deals with the Search array only, accessing it through DB_GetNfo.
;============================================================================




;----------------------------------------------------------------------------
ShowTime	proc	uses ebx
;-------
;
; Draw Time window into a back buffer and blit to screen to avoid flicker.
;
;
; Entry		External:
;
;		BaseItem	Base Item of display
;
;		zAdj		Current adjustment factor for z value, +/-
;				Modified by user interaction.
;
;		SrchItems	number of items in the search array
;
;
; Exit		BaseItem	may be updated
;
;		zAdj		unchanged
;
;		SrchItems	unchanged
;
;		CurrItem	current first item in Time display
;
;
; Time window item display
;
; A sliding block of DispItems items is displayed beginning from a base
; item which has the start index into the search array.	Items are drawn from
; older to newer so newer items overlay older.
;
;  Newest Item		Search Array (SrchItems items)		Oldest Item
;	|							    |
;	|-------+-------------------------------+-------------------|
;	0	|				|		SrchItems-1
;	      <-|--------- Display block -------|->
;		|	 (DispItems items)	|
;		^				^
;	     BaseItem	  		BaseItem + (DispItems-1)
;
;			+ <- zAdj -> -
;
; The z coordinate of an item is calculated by adding its index, + 1, to a z
; adjustment factor set by the user's mouse movements. The z coordinate
; is used to calculate the item's x and y screen coordinates.
;
; Incrementing the z adjustment moves the items away giving the appearance
; of moving forward in time. If the oldest displayed item moves off the top
; of the screen then BaseItem is decremented to adjust the display block
; down the search array.
;
; Decrementing the z adjustment moves the items closer giving the appearance
; of moving backward in time. If the newest displayed item moves off the
; bottom of the screen then BaseItem is incremented to adjust the displayed
; block up the search array.
;
; Note that the highest value of BaseItem is SrchItems - DispItems because
; BaseItem + (DispItems - 1) = SrchItems - 1
;
; If a search returns fewer than the default number of displayed items then
; the displayed item count is set to the number of items found in the search.
;
;
; The time taken to draw a Time window is dominated by image thumbnailing as
; can be seen if ShowDTTime is enabled and thumbnailing is toggled on/off.
;----------------------------------------------------------------------------


	local	DrawNdx		:dword		; draw index
	local	x		:dword		; x, integer
	local	y		:dword		; y, integer
	local	z		:dword		; z, floating point
	local	zi		:dword		; z, integer (rnd to nearest)

	local	hOBitmap	:dword		; handle, original Bitmap
	local	hOBkGnd		:dword		; handle, original Background
	local	hOFont		:dword		; handle, original Font
	local	hOBrush		:dword		; handle, original Brush
	local	szItemNum[32]	:byte		; Item number and total


if ShowDTTime	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		counter_begin 1, HIGH_PRIORITY_CLASS
endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


; Clear back buffer to background color.


	invoke	SelectObject, hMemDC, hBitmap
	mov	hOBitmap, eax
	invoke	FillRect, hMemDC, addr ScrnRect, hBGBrush


; Draw Time data into the back buffer.


	; Set Item fill color, font and text background color.

	invoke	SelectObject, hMemDC, hItemBrush
	mov	hOBrush, eax
	invoke	SelectObject, hMemDC, hFont
	mov	hOFont, eax
	invoke	SetBkColor, hMemDC, CTimeItem
	mov	hOBkGnd, eax


	; ===================================================================
	; Draw Items from oldest to newest.
	; ===================================================================


	; At entry to the loop DrawNdx is set to BaseItem + DispItems + 1.
	; The added 1 compensates for the first decrement of DrawNdx
	; so DrawNdx is effectively the Item index + 1.
	;
	; We exit the loop with DrawNdx set to the newest visible Item
	; index + 1.

	mov	eax, nLgItems			; Initialise LI index
	mov	LINdx, eax			; for DrawItem.

	mov	ecx, DispItems
	mov	eax, BaseItem
	add	eax, ecx
	inc	eax				; (compensate for first entry)
	mov	DrawNdx, eax


	if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_DT_Entry, hMemDC, SrchItems, DispItems,
					BaseItem, DrawNdx, zAdj
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	.while	(sdword ptr ecx > 0)


		dec	DrawNdx


		; z = (Item index + 1) + adjustment factor.

		fild	DrawNdx
		fadd	zAdj
		fst	z
		fist	zi			; integer Z for DrawItem


		if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_DT_NdxZi, DrawNdx, zi
		endif		;++++++++++++++++++++++++++++++++++++++++++++


		; If z is <= 1 then the item is off the screen at the
		; bottom so move the display block up. If the display
		; block is at the end of the array then freeze it:
		;
		;	BaseItem = SrchItems - DispItems
		;
		; Also as this item is not drawn we need to bump the
		; DrawNdx back up to the last visible item.

		pushd	1
		ficom	dword ptr [esp]		; z - 1
		fstsw	[esp]			; fpu flags to stack
		fwait				; ensure op is complete
		pop	eax			; reset stack, eax has flags
		sahf				; fpu flags to cpu flags
		ja	@@Zgt1			; z > 1

		; z <= 1

		fstp	x 			; (pop st0, x not yet req'd)

		inc	BaseItem
		mov	edx, BaseItem
		add	edx, DispItems
		mov	eax, SrchItems

		.if	(edx >= eax)

			sub	eax, DispItems

			; Make very sure our BaseItem is > 0
			; or a crash is inevitable!.
			;
			;[Not necessary but fool with things and]
			;[this is generally where it happens.   ]
			;
			;.if	(sdword ptr eax < 0)
			;	xor	eax, eax
			;.endif

			mov	BaseItem, eax

		.endif


		inc	DrawNdx


		if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_DT_Zu1, hMemDC, BaseItem
		endif		;++++++++++++++++++++++++++++++++++++++++++++


		; We can exit now as there are no more items to draw as
		; it/they are all off the bottom of the screen.

		jmp	@@ExitLoop


		@@Zgt1:

		; If z > DefDispItems + 1 then an item has effectively moved
		; off the top of the screen so move the display block down
		; the search array by decrementing the BaseItem. If the
		; display block is at the beginning of the array then
		; freeze it:	BaseItem <- 0

		mov	eax, DefDispItems
		inc	eax
		push	eax
		ficomp	dword ptr [esp]		; z - (DispItems + 1)
		fstsw	[esp]			; fpu flags to stack
		fwait				; ensure op is complete
		pop	eax			; reset stack, eax has flags
		sahf				; fpu flags to cpu flags
		jb	@@ZleDI			; z < DispItems + 1
		jz	@@ZleDI			; z = DispItems + 1

		; z > DispItems + 1

		mov	eax, BaseItem

		.if	(eax != 0)
			dec	eax
		.endif

		mov	BaseItem, eax


		if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_DT_ZgtDI, hMemDC, BaseItem
		endif		;++++++++++++++++++++++++++++++++++++++++++++


		; No need to draw this item but we do need to draw the rest!.

		jmp	@@LoopContinue


		@@ZleDI:

		; Calculate x and y coordinates.

		; x2D = x/z * screen_width + screen_width/2

		mov	eax, DrawNdx		; Get Right x for
		and	eax, 1			; even index and
		shl	eax, 2			; Left x for odd.
		fld	[xI_R + eax]		; (see .data!)

		fdiv	z
		fimul	G_ScrnHgt
		fiadd	xMid
		fistp	x

		; y2D = screen_height / z	(1/z * screen_height)

		fild	G_ScrnHgt
		fdiv	z
		fistp	y


		if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++
				invoke	Dbg_DT_XY, hMemDC, x, y
		endif		;++++++++++++++++++++++++++++++++++++++++++++


		; Draw the item.

		push	ecx
		invoke	DrawItem, x, y, zi, hMemDC, DrawNdx
		pop	ecx


		@@LoopContinue:

		dec	ecx


	.endw


	@@ExitLoop:

	; ===================================================================


	if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_DT_Exit, hMemDC, SrchItems, DispItems,
					BaseItem, DrawNdx
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	; ===================================================================
	; Draw Time boxes and status.
	; ===================================================================


	; The strings are padded with a space at beginning and end.

	invoke	SetBkColor, hMemDC, CTimeBox


	; End Time ----------------------------------------------------------
	;
	; Item SrchItems-1 is last item in Database.

	mov	eax, SrchItems
	dec	eax

	invoke	DB_GetNfo, I_GETTMSP, eax, addr EndTime.TSStrg[1]
	mov	dword ptr EndTime.TSStrg[20], ' '

	invoke	DrawTextA, hMemDC, addr EndTime.TSStrg, -1, addr EndTime,
			DT_NOCLIP or DT_SINGLELINE


	; Start Time --------------------------------------------------------
	;
	; Item 0 is first item in Database.

	invoke	DB_GetNfo, I_GETTMSP, 0, addr StaTime.TSStrg[1]
	mov	dword ptr StaTime.TSStrg[20], ' '

	invoke	DrawTextA, hMemDC, addr StaTime.TSStrg, -1, addr StaTime,
			DT_NOCLIP or DT_SINGLELINE

	; Temporarily store the 1 based index of our last item drawn for
	; number of Items box, ensuring it is not greater than the number
	; of items in the array.

	mov	eax, DrawNdx

	.if	(eax > SrchItems)
		mov	eax, SrchItems
	.endif

	mov	DrawNdx, eax			; 1 -> SrchItems


	; Move Time ---------------------------------------------------------
	;
	; Position the Move Time box to a y coordinate corresponding
	; to the newest visible item.
	;
	;	y1 = ((index/Items) * Y)
	;	y = Y - y1	(convert to screen coordinate)
	;
	; Fudge: As our DrawNdx ranges from 1 to SrchItems it will offset
	; item index 0 up from the Start Time box. We will decrement it so
	; it is on the Start Time box but now will be offset below the End
	; Time box for the last item with low SrchItems, ce la vie.
	; Incidently this provides a correct index for the database.

	dec	eax
	mov	CurrItem, eax			; (store for MoveTime Up/Dn)
	mov	SA_CItem, eax			; (store for Time_GetItem)

	push	eax

	mov	x, eax				; (temp store for fpu)
	fild	x
	fidiv	SrchItems
	fimul	TMRange				; Y
	fistp	y				; y1
	mov	eax, TMRange
	sub	eax, y				; Y - y1

	mov	MovTime.top, eax
	add	eax, TBoxHgt_1
	mov	MovTime.bottom, eax

	pop	eax				; 0 -> SrchItems-1 for
						; Database access

	invoke	DB_GetNfo, I_GETTMSP, eax, addr MovTime.TSStrg[1]
	mov	dword ptr MovTime.TSStrg[20], ' '

	invoke	DrawTextA, hMemDC, addr MovTime.TSStrg, -1, addr MovTime,
			DT_NOCLIP or DT_SINGLELINE or DT_VCENTER

	; Have to use DrawIconEx so Windows doesn't scale out to 32x32!.
	; [need to calc and set rectangle x for Icon drawing alongside]

	mov	eax, MovTime.right
	mov	edx, MovTime.top
	push	eax
	push	edx
	invoke	DrawIconEx, hMemDC, eax, edx, hDnIcon,
			0, 0, NULL, NULL, DI_MASK
	pop	edx
	pop	eax
	add	eax, MoveIcoXY
	invoke	DrawIconEx, hMemDC, eax, edx, hUpIcon,
			0, 0, NULL, NULL, DI_MASK


	; Build and show Item and number of Items ---------------------------
	;
	; Quicker to build text strings rather than cat from data.

	invoke	SetBkColor, hMemDC, CTimeItem
	invoke	SelectObject, hMemDC, hFontB

	lea	ebx, szItemNum
	push	ebx
	mov	eax, 'etI '
	mov	[ebx], eax
	mov	eax, '  m'
	mov	[ebx + 4], eax
	add	ebx, 6
	invoke	dwtoa, DrawNdx, ebx
	invoke	StrLen, addr szItemNum
	pop	ebx
	push	ebx
	add	ebx, eax
	mov	eax, ' fo '
	mov	[ebx], eax
	add	ebx, 4
	invoke	dwtoa, SrchItems, ebx
	invoke	StrLen, addr szItemNum
	pop	ebx
	add	ebx, eax
	mov	ecx, ' '			; space + terminator
	mov	edx, G_UDBStatus

	.if	(edx & UDBDbStale)
		mov	eax, ')?( '		; database is stale
		mov	[ebx], eax
		add	ebx, 4
	.endif

	mov	[ebx], cx

	invoke	DrawTextA, hMemDC, addr szItemNum, -1, addr ItemsBox,
			DT_NOCLIP or DT_SINGLELINE


	; ===================================================================


; Blit back buffer to screen.


	mov	eax, ScrnRect.right
	sub	eax, ScrnRect.left
	mov	edx, ScrnRect.bottom
	sub	edx, ScrnRect.top
	invoke	BitBlt, hDC, ScrnRect.left, ScrnRect.top, eax, edx,
			hMemDC, 0, 0, SRCCOPY


; Replace the original objects.


	invoke	SelectObject, hMemDC, hOBrush
	invoke	SelectObject, hMemDC, hOFont
	invoke	SetBkColor, hMemDC, hOBkGnd
	invoke	SelectObject, hMemDC, hOBitmap


if ShowDTTime	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		counter_end
		PrintDec eax, "Show"
endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	ret


ShowTime	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
DrawItem	proc	uses ebx esi edi,
			x:dword,y:dword,zi:dword,hDCi:dword,Item:dword
;-------
;
; Draw Item at supplied coordinates.
;
;
; Entry		x,y		screen coordinates of top left of item
;
;		zi		z coordinate, integer
;
;		hDCi		Device Context
;
;		Item		1 based index of Item in search array
;
;
; Exit		n/a
;
;
; NB Large Item code depends on the LIstruc being 20 bytes in size!.
;
; Note scaling images and storing the scaled versions speeds up the GDI+
; section about 5-10x.
;----------------------------------------------------------------------------


	local	xSize			:dword	; Item/Image width
	local	ySize			:dword	; Item/Image height
	local	hOFontB			:dword	; handle, original Font bold
	local	hOText			:dword	; handle, original text color
	local	TxtBox			:RECT	; coordinates of Item box
	local	nBytesRd		:dword	; bytes read, text thumb
	local	hImage			:dword	; handle, loaded image
	local	szIPath[MAX_PATH]	:byte	; ascii Item file path
	local	szIPathW[MAX_PATH*2]	:byte	; unicode Item file path


; Size based on distance from viewer.


	.if	(zi < ItemLrgZ)			; Large size

		mov	xSize, ItemWidL
		mov	ySize, ItemHgtL

	.elseif	(zi < ItemVarZ)			; Variable size

		fild	zi
		fdiv	zKVarItem		; perspective constant
		fistp	xSize			; (temp var)
		mov	eax, xSize
		mov	edx, ItemWidL/2
		sub	edx, eax
		mov	ecx, edx
		mov	xSize, edx
		mov	ySize, ecx

	.else 	; (ItemVarZ >= z <= DefDispItems) = Smallest size

		invoke	SetPixelV, hDCi, x, y, CTimeItem
		jmp	@@Exit

        .endif


; Build rectangle dimensions, adjust it over to the left if it is left
; of midscreen, and draw it.


	mov	eax, xMid

	.if	(x > eax)			; Right

		mov	eax, x
		add	eax, xSize
		mov	edx, y
		add	edx, ySize

	.else					; Left

		mov	eax, x
		mov	ecx, eax
		sub	ecx, xSize
		mov	x, ecx

		mov	edx, y
		add	edx, ySize

	.endif

	push	eax

	invoke	Rectangle, hDCi, x, y, eax, edx

	pop	eax				; RECT.right


; Draw Item details only if Item is large.


	.if	(zi < ItemLrgZ)


		; Save large Item rectangle for detection.
		; Save from farthest to nearest.

		mov	ecx, LINdx
		dec	ecx			; 0 based array
		mov	LINdx, ecx

		lea	ecx, [ecx + ecx*4]	; Item * 5
		shl	ecx, 2			;  * 4 = sizeof LIstruc

		mov	LI[ecx].Coord.right, eax
		mov	LI[ecx].Coord.bottom, edx
		mov	eax, x
		mov	LI[ecx].Coord.left, eax
		mov	eax, y
		mov	LI[ecx].Coord.top, eax

		;== Database indexes from 0 up (NB from now on) ==

		dec	Item

		mov	eax, Item
		mov	LI[ecx].Ndx, eax


		; Draw File icon, but only if loaded.

		add	x, ItemIcoX
		add	y, ItemIcoY
		invoke	DB_GetNfo, I_GETICON, Item, NULL

		.if	(eax)
			invoke	DrawIcon, hDCi, x, y, eax
		.endif


		; Draw Time stamp.
		;
		; Use szIPath as a temporary string variable.

		invoke	SetTextColor, hDCi, CTimeStamp
		mov	hOText, eax
		invoke	SelectObject, hDCi, hFontB
		mov	hOFontB, eax

		add	x, ItemTsX
		invoke	DB_GetNfo, I_GETTMSP, Item, addr szIPath
		invoke	StrLen, addr szIPath
		invoke	TextOutA, hDCi, x, y, addr szIPath, eax

		invoke	SelectObject, hDCi, hOFontB
		invoke	SetTextColor, hDCi, hOText


		; Draw File name.

		add	y, ItemTxtY1
		invoke	DB_GetNfo, I_GETNAME, Item, NULL
		mov	esi, eax

		push	esi			; save Name pointer

		mov	eax, x
		mov	TxtBox.left, eax
		add	eax, ItemThmWid - ItemTsX
		mov	TxtBox.right, eax
		mov	eax, y
		mov	TxtBox.top, eax
		add	eax, ItemTxtY2
		mov	TxtBox.bottom, eax
		invoke	DrawTextA, hDCi, esi, -1, addr TxtBox,
				DT_LEFT or DT_END_ELLIPSIS


		; Draw File Path.

		sub	x, ItemTsX - ItemIcoX
		add	y, ItemTxtY2
		invoke	DB_GetNfo, I_GETPATH, Item, NULL
		mov	esi, eax

		push	esi			; save Path pointer

		mov	eax, x
		mov	TxtBox.left, eax
		add	eax, ItemThmWid
		mov	TxtBox.right, eax
		mov	eax, y
		mov	TxtBox.top, eax
		add	eax, ItemTxtY2
		mov	TxtBox.bottom, eax
		invoke	DrawTextA, hDCi, esi, -1, addr TxtBox,
				DT_LEFT or DT_PATH_ELLIPSIS


		pop	edi			; Path
		pop	esi			; Name


		;============================================================
		; Item Thumbnailing.
		;============================================================


		; Do we want thumbnails?.

		.if	((zi >= ItemThumb) || (DoThumb == 0))
			jmp	@@Exit		; no
		.endif


		; Does it have a handler?.

		invoke	DB_GetNfo, I_GETTHUM, Item, NULL

		.if	(eax == 0)		; no
			jmp	@@Exit
		.endif

		mov	ebx, eax		; Handler ID


		; Make full path and check file exists.
		;
		; We may have deleted files before a database update.

		invoke	szCopy, edi, addr szIPath
		invoke	szCatStr, addr szIPath, esi
		invoke	File_Exist, addr szIPath

		.if	(CARRY?)		; doesn't exist
			jmp	@@Exit
		.endif


		; Ok now do it.

		add	y, ItemThmOff		; set y


		.if	(ebx == FTH_Img)


			; Don't go there if GDI+ disabled or not loaded. We
			; need to separately test in here so the FTH_Img test
			; is taken - else a crash at the .else below!.

		 	.if	(G_GdipOk == 0)
				jmp	@@Exit
			.endif


			; Image Thumbnail (built-in) ------------------------


			; Check if this Item's image is loaded already.

			mov	eax, Item
			mov	ebx, eax
			and	ebx, nImgObj-1		; Index
			shl	ebx, ImgObjRecMult	; * bytes per object
			cmp	eax, ImgObj[ebx].Item	; = Object
			je	@@FoundImage


			; Not loaded, store Item.

			mov	ImgObj[ebx].Item, eax


			; Make a unicode version of the full path.

			invoke	MultiByteToWideChar, CP_ACP, 0, addr szIPath,
					-1, addr szIPathW, MAX_PATH*2


			; Load the image, get its dimensions and scale it
			; into the graphics object then release it as we
			; don't need it any more. We also need to clear the
			; graphics object background.

			invoke	GdipLoadImageFromFile, addr szIPathW,
					addr hImage

			invoke	GdipGetImageWidth, hImage, addr xSize
			invoke	GdipGetImageHeight, hImage, addr ySize
			mov	eax, xSize
			shl	eax, 16
			add	eax, ySize
			mov	ImgObj[ebx].ImgXY, eax

			invoke	GdipGraphicsClear, ImgObj[ebx].hGra, CBlack
			invoke	GdipDrawImageRectI, ImgObj[ebx].hGra, hImage,
					0, 0, ItemThmWid-1, ItemThmHgt-1
			invoke	GdipDisposeImage, hImage


			; Draw image.
			;
			; Note GDI+ RECT structure is left,top,width,height.

			@@FoundImage:

			invoke	GdipDrawImageRectI, hGdipGraObj,
					ImgObj[ebx].hBmp,
					x, y, ItemThmWid, ItemThmHgt


		.elseif	(ebx == FTH_Txt)


			; Text Thumbnail (built-in) -------------------------


			; Check if this Item's thumbnail is loaded already.

			mov	eax, Item
			mov	ebx, eax
			and	ebx, nTxtObj-1		; Index,

			mov	esi, ebx
			shl	esi, nCharTTMult	; (calc ptr to object
			add	esi, offset TxtObjArr	;  in object array)

			shl	ebx, TxtObjRecMult	; * bytes per object,
			cmp	eax, TxtObj[ebx].Item	; = Object.
			je	@@FoundText


			; Read the first nCharTxtThumb bytes of data, or all
			; of the file if size is < nCharTxtThumb bytes.

			mov	TxtObj[ebx].Item, eax

			invoke	File_ReadCl, addr szIPath, esi,
					nCharTxtThumb, FILE_ATTRIBUTE_NORMAL

			.if	((CARRY?) || (eax == 0))

				mov	TxtObj[ebx].Item, -1
				jmp	@@Exit		; error or no data

			.endif

			mov	TxtObj[ebx].Len, eax


			; Show the Synopsis (esi has pointer).

			@@FoundText:

			mov	ebx, TxtObj[ebx].Len
			push	dword ptr offset @@Exit
			jmp	@@DrawText


		.else


			; Plugin Handler ------------------------------------


			invoke	DB_GetThumData, ebx, T_GETFUNC

			.if	((CARRY?) || (eax == 0))
				jmp	@@Exit		; No handler or zero
			.endif				;  pointer.

			push	Item
			lea	edx, szIPath
			push	edx
			call	eax

			.if	(eax)			; ensure pointer!

				mov	ebx, eax
				mov	esi, edx
				push	dword ptr offset @@Exit
				jmp	@@DrawText

			.endif


		.endif

		;============================================================


	.endif


	@@Exit:


	if ShowTDisp	;++++++++++++++++++++++++++++++++++++++++++++++++++++
			invoke	Dbg_DrawItem, hMemDC, Item, x, y, zi, xSize,
					ySize
	endif		;++++++++++++++++++++++++++++++++++++++++++++++++++++


	ret


@@DrawText:
;----------
;
; Local subroutine to draw text in Item.
;
; Entry		esi	address of text
;		ebx	length of text
;		[esp]	return label


	mov	eax, x
	mov	TxtBox.left, eax
	add	eax, ItemThmWid
	mov	TxtBox.right, eax
	mov	eax, y
	mov	TxtBox.top, eax
	add	eax, ItemThmHgt
	mov	TxtBox.bottom, eax

	invoke	SetBkColor, hDCi, CWhite
	invoke	FillRect, hDCi, addr TxtBox, hWhiteBrush
	invoke	DrawTextA, hDCi, esi, ebx, addr TxtBox,
			DT_LEFT or DT_WORDBREAK or DT_EXPANDTABS
	invoke	SetBkColor, hDCi, CTimeItem

	pop	eax
	jmp	dword ptr eax


DrawItem	endp
;----------------------------------------------------------------------------




;============================================================================
